/*************************************************************************

This is a "far from perfect" converter that takes gnuplot table output
and splits it in polygons. Then it calls sipp to render it.
You get sipp from isy.liu.se:/pub/sipp or ask archie.
try this:
set samples 40
set isosamples 40
splot sin(x)*sin(y)/x/y
set output "out.table"
set term table
replot

then go back to the shell and do:
gnuconv -s 400 <out.table

and:
xv out.ppm

This software uses the sipp library. So you have to install it first.
samples and isosamples have to be the same. 

Options in gnuconv are:
-l for line rendering; similar to what gnuplot does.
-p for phong shading; default
-g gouraud shading
-f flat shading 
-s size; the x and y size of the rendered image
-P for ppm output
-X for output in X11 window

ciao
Carsten Hammer
chammer@POST.uni-bielefeld.de 

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sipp.h>
#include <primitives.h>

#define X11 1
#define PPM 0
extern char *optarg;

main(int argc,char **argv)
{
double (*feld)[10][3];
char st[80],*dum;
char mylinebuffer[80];
double xx,yy,zz;
double xmax,xmin,ymax,ymin,zmax,zmin;
double xmid,ymid,zmid,xsize,ysize,zsize;
int i,ii=0,feldgroesse,zeile,spalte;
FILE *image_fd;
int ru;
Surface * mysurface;
Object * myobjekt;

int mode,output;
int c,block;
int size;

sipp_init();
mode=PHONG;
output=PPM;
size=256;

		/* shameless stolen from sipp demos 	*/

while ((c=getopt(argc,argv,"XPpgfls:")) !=EOF)
  {
    switch (c) {
          case 'X':
           output = X11;
            break;

          case 'P':
           output = PPM;
            break;

	  case 'p':
           mode = PHONG;
            break;

          case 'g':
            mode = GOURAUD;
            break;

          case 'f':
            mode = FLAT;
            break;

          case 'l':
            mode = LINE;
            break;

          case 's':
            size = atoi(optarg);
            break;
	 }
  }

	/*	reading the gnuplot table output	*/
	/*	seems to make problems on some platforms */

feld=malloc(sizeof(double)*3*10);
block=10;	/*	after 10 coords. reallocing	*/
for(;0==feof(stdin);)
 {
  fgets(st,80,stdin);  /*	this is not dynamical yet	*/
  if (st[0]=='i') 
   {
     strcpy(mylinebuffer,(char *)strtok(st,"=")); 
     if (dum=strtok(NULL,"y="))
       {
     xx=strtod((char *)dum,(char **)NULL);
     yy=strtod((char *)strtok(NULL,"z="),(char **)NULL);
     zz=strtod((char *)strtok(NULL,"= "),(char **)NULL);
     fflush(stdout);
	if (!block) {feld=realloc(feld,sizeof(double)*3*(ii+10));block=10;} 
     (*feld)[ii][0]=xx;
     (*feld)[ii][1]=yy;
     (*feld)[ii][2]=zz;
     ii++;                /*       zaehlt die Anzahl der Koordinaten    */
	block--;
       }
    }
 }

                   /*         find the max and min values              */

xmax=xmin=(*feld)[0][0];
ymax=ymin=(*feld)[0][1];
zmax=zmin=(*feld)[0][2];
for(i=1;i<ii;i++)
  {
    if((*feld)[i][0]<xmin) xmin=(*feld)[i][0];
    if((*feld)[i][1]<ymin) ymin=(*feld)[i][1];
    if((*feld)[i][2]<zmin) zmin=(*feld)[i][2];
    if((*feld)[i][0]>xmax) xmax=(*feld)[i][0];
    if((*feld)[i][1]>ymax) ymax=(*feld)[i][1];
    if((*feld)[i][2]>zmax) zmax=(*feld)[i][2];
  }
xmid=(xmin+xmax)/2;
ymid=(ymin+ymax)/2;
zmid=(zmin+zmax)/2;
xsize=20/(xmax-xmin);
ysize=20/(ymax-ymin);
zsize=20/(zmax-zmin);


                  /*      move and resize the thing                 */
                  /*  not very elegant to move all coordinates      */
                  /*  instead of moving camera but who cares..       */

for(i=0;i<ii;i++)
  {
    (*feld)[i][0]=((*feld)[i][0]-xmid)*xsize;
    (*feld)[i][1]=((*feld)[i][1]-ymid)*ysize;
    (*feld)[i][2]=((*feld)[i][2]-zmid)*zsize;
  }

lightsource_create(1.0,1.0,1.0, 1.0,1.0,1.0, LIGHT_DIRECTION);

image_fd=fopen("out.ppm","w");
sipp_show_backfaces(TRUE);

feldgroesse=(int)(sqrt((double)ii/2));
/* printf("Koordinatenanzahl=%i,Feld ist %i**2 in der Eingabedatei!\n",ii,feldgroesse);
printf("Koordinatenanzahl=%i des statischen Eingabefeldes!\n",ii/2);*/

	/*	building the function surface	*/

for (zeile=1;zeile<=feldgroesse-1;zeile++)
  {
   for (spalte=1;spalte<=feldgroesse-1;spalte++)
     {
       ru=zeile*feldgroesse+spalte;
       vertex_push((*feld)[ru][0],(*feld)[ru][1],(*feld)[ru][2]);
       vertex_push((*feld)[ru-1][0],(*feld)[ru-1][1],(*feld)[ru-1][2]);
       vertex_push((*feld)[ru-1-feldgroesse][0],
	(*feld)[ru-1-feldgroesse][1],
	(*feld)[ru-1-feldgroesse][2]);
       vertex_push((*feld)[ru][0],(*feld)[ru][1],(*feld)[ru][2]);
       polygon_push();
       vertex_push((*feld)[ru][0],(*feld)[ru][1],(*feld)[ru][2]);
       vertex_push((*feld)[ru-feldgroesse][0],
	(*feld)[ru-feldgroesse][1],
	(*feld)[ru-feldgroesse][2]);
       vertex_push((*feld)[ru-1-feldgroesse][0],
	(*feld)[ru-1-feldgroesse][1],
	(*feld)[ru-1-feldgroesse][2]);
       vertex_push((*feld)[ru][0],(*feld)[ru][1],(*feld)[ru][2]);
       polygon_push();
     }  
 }
mysurface= surface_basic_create(0.4 ,0.5,0.5,0.8 ,0.6 ,0.1 ,1.0,1.0,1.0);
myobjekt=object_create();
object_add_surface(myobjekt,mysurface);
object_add_subobj(sipp_world,myobjekt);
camera_params(sipp_camera,20.0,100.0,100.0 ,0.0,0.0,0.0 ,0.0,0.0,1.0 ,0.1);
printf("Rendering, wait...\n");
fflush(stdout);
switch (output)
 {
	case	PPM:
	  render_image_file(size,size,image_fd,mode,1);
	  break;

	case	X11:
	  render_ximage(size,size,mode,1,0,0,0);
	/*	this isnt elegant too..	*/
	  sleep(5);
	  break;
 }
return(0);
}









